#!/bin/bash
source /etc/stable-arch/config

currentdate=$(date +%y%m%d%H)
currentroot="$(mount | awk '/root.* \/ / {print $1}')"
if [[ $encryptedroot == "true" && $(zfs get -H -o value keystatus "$currentroot") != "available" ]]
then
	echo "You have not loaded key for $currentroot"
	exit
fi
[[ $bootpool == "true" ]] && currentboot="$(zfs get mountpoint | grep "$(readlink -f /boot)" | awk '{print $1}')"
[[ $bootpool == "true" ]] && nextboot="$bootzpool/boot_$currentdate"
nextroot="$rootzpool/root_$currentdate"

rm -rf /var/cache/pacman/pkg/*xz
rm -rf /var/cache/pacman/pkg/*zst

zfs snapshot "$currentroot"@root_"$currentdate" || { echo could not snapshot "$currentroot"@root_"$currentdate" ; exit 1 ; }
zfs clone "$currentroot"@root_"$currentdate" -o canmount=on -o mountpoint=/"$nextroot" "$nextroot" || { echo could not clone "$currentroot"@root_"$currentdate" "$nextroot" ; exit 1 ; }
if [[ $bootpool == "true" ]] 
then 
	zfs snapshot "$currentboot"@boot_"$currentdate" || { echo could not snapshot "$currentboot"@boot_"$currentdate" ; exit 1 ; }
	zfs clone "$currentboot"@boot_"$currentdate" -o canmount=on -o mountpoint=/"$nextboot" "$nextboot" || { echo could not clone "$currentboot"@boot_"$currentdate" "$nextboot" ; exit 1 ; }
	rm /"${nextroot:?}"/boot -r
	mkdir /"${nextroot:?}"/boot
	mount -o bind /"$nextboot" /"$nextroot"/boot
fi

mount -o bind /home /"$nextroot"/home

#should i even check for workdir? just put all that shit in $PATH or something
workdir="$( cd "$( dirname "${BASH_SOURCE[0]}" )" >/dev/null 2>&1 && pwd )"
"$workdir"/systemupdate.expect "$sudouser" "/$nextroot" "$packagemanager"

if [[ $bootpool == "true" ]] 
then 
	umount /"$nextroot"/boot
	rm -r /"${nextroot:?}"/boot
	ln -s /"$nextboot" /"$nextroot"/boot
fi

if [[ -f /boot/grub/grub.d/root_"$currentdate".cfg ]]
then
	echo "grub file already exists, are you running it second time?"
else
	echo "source \$prefix/grub.d/root_$currentdate.cfg" >> /boot/grub/grub.cfg
	cp /boot/grub/grub.d/root_template /boot/grub/grub.d/root_"$currentdate".cfg
	sed -i "s/_template/_$currentdate/g" /boot/grub/grub.d/root_"$currentdate".cfg
	sed -i "s/arch-root.*/arch-root_$currentdate/g" /boot/grub/grubenv
fi


sleep 2
umount -lq /"$nextroot"/*

#post update maintenance
read -r -p "Delete old clones?(y/N) " -e dialog
if [[ $dialog == y ]]
then
	echo "Clones to remove:"
	removepools=$(zfs list -o name| grep "$rootzpool/" | grep -E -v "$currentroot" | fzf -m --height=50% --reverse)
	removename=$(echo "$removepools" | sed -e "s,$rootzpool/,,g")

	# promote currentroot or else zfs will not let us remove old filesystems
	zfsorigin=$(zfs get origin -o value -H "$currentroot")
	while [[ $zfsorigin != "-" ]]
	do
		#debug
		echo "zfs promote $currentroot"
		zfs promote "$currentroot"
		[[ $bootpool == "true" ]] && zfs promote "$currentboot"
		zfsorigin=$(zfs get origin -o value -H "$currentroot")
	done

	for i in $removepools
	do
		[[ $bootpool == "true" ]] && b=$(echo "$i" | sed "s~$rootzpool/root_~$bootzpool/boot_~g")
		[[ $bootpool == "true" ]] && zfs list -o name| grep "$b" | xargs -n1 zfs destroy -r
		zfs list -o name| grep "$i" | xargs -n1 zfs destroy -r
	done

	# remove snapshots from old datasets
	for i in $removename
	do 
		[[ $bootpool == "true" ]] && b=$(echo "$i" | sed 's/root/boot/g')
		[[ $bootpool == "true" ]] && zfs list -t snapshot -o name| grep "@$b" | xargs -n1 zfs destroy
		zfs list -t snapshot -o name| grep "@$i" | xargs -n1 zfs destroy
	done

	# remove grub entries
	for i in $removename
	do
		if [[ -f /boot/grub/grub.d/"$i".cfg ]]
		then
			rm /boot/grub/grub.d/"$i".cfg
			sed -i "/$i/d" /boot/grub/grub.cfg
		else 
			echo "$i".cfg does not exist, this should not have happened, shit is broken yo
		fi
	done

else exit
fi
